/* REVISION HISTORY

2a_1 -- rewrote to handle screen resizing and made main file work "properly" -- may
          be more stable!
        added revision history notes to code
        added "reset" button (esc) and code in backend to reset
        
2a   -- initial windows release, added #defines to fractroam.c to handle windows
          functions

*/


#include <windows.h>

#include "fractroam.h"

LRESULT CALLBACK WndProc( HWND hWnd, UINT messg,
								WPARAM wParam, LPARAM lParam );


char szProgName[] = "WinFractRoam 2a-1"; /* main window title */

HBITMAP hbit; // bitmap
HDC memdc;

BOOL ready=FALSE;

int MYWIDTH,MYHEIGHT;
								
int WINAPI WinMain( HINSTANCE hInst, 	/*Win32 entry-point routine */
					HINSTANCE hPreInst, 
					LPSTR lpszCmdLine, 
					int nCmdShow )
{
	HWND hWnd;
	MSG lpMsg;
	WNDCLASS wc;

	if( !hPreInst )			/*set up window class and register it */
	{
		wc.lpszClassName 	= szProgName;
		wc.hInstance 		= hInst;
		wc.lpfnWndProc		= WndProc;
		wc.hCursor			= LoadCursor( NULL, IDC_ARROW );
		wc.hIcon			= LoadIcon( NULL, IDI_APPLICATION );
		wc.lpszMenuName		= NULL;
		wc.hbrBackground	= GetStockObject( WHITE_BRUSH );
		wc.style			= 0;
		wc.cbClsExtra		= 0;
		wc.cbWndExtra		= 0;

		if( !RegisterClass( &wc ) )
			return FALSE;
	}


	hWnd = CreateWindow( 	szProgName,							/* now create the window */
							szProgName,
							WS_OVERLAPPEDWINDOW,
							CW_USEDEFAULT, // X
							CW_USEDEFAULT, // Y
							DEFAULT_FRACT_WIDTH,
							DEFAULT_FRACT_HEIGHT,
							(HWND)NULL, //handle to "parent"
							(HMENU)NULL, // no menu
							(HANDLE)hInst, // handle to "us"
							(LPSTR)NULL		);

	ShowWindow(hWnd, nCmdShow );
	UpdateWindow( hWnd );
	
	while( GetMessage( &lpMsg, NULL, 0, 0 ) )					/* begin the message loop */
	{
		TranslateMessage( &lpMsg );
		DispatchMessage( &lpMsg );
	}
	return( lpMsg.wParam);
}

void quit(HWND hwnd) {
	KillTimer(hwnd,1);
  	PostQuitMessage( 0 );

}

LRESULT CALLBACK WndProc( HWND hWnd, UINT messg,				/*callback procedure */
								WPARAM wParam, LPARAM lParam )
{
	HDC hdc; 				/* handle to the device context */
	PAINTSTRUCT pstruct;
	RECT rect;
	BOOL replot=FALSE;
	char coordinates[360];
	int x,y,i,width,height;
	short* fractal;
	switch(messg)
	{
		case WM_SIZE:
			GetWindowRect(hWnd,&rect);
			MYWIDTH=rect.right-rect.left;
			MYHEIGHT=rect.bottom-rect.top;
			replot=TRUE;
		break;
			
		case WM_TIMER: // stuff
			fract_update();
			replot=TRUE;
		break;

		case WM_LBUTTONDOWN:
			fract_center((int)(((double)DEFAULT_FRACT_WIDTH/(double)MYWIDTH)*LOWORD(lParam)),			(int)(((double)DEFAULT_FRACT_HEIGHT/(double)MYHEIGHT)*(MYHEIGHT-HIWORD(lParam))));
			replot=TRUE;
		break;
		
		case WM_RBUTTONDOWN:
			fract_pilot_toggle();
		break;
		
		case WM_KEYDOWN:
			switch(wParam) {
				case 81: quit(hWnd); break; // q
				case VK_LEFT:
				case 72: fract_move( 0,-1, 0); replot=TRUE; break; // h
				case VK_RIGHT:
				case 76: fract_move( 0, 1, 0); replot=TRUE; break; // l
				case VK_DOWN:
				case 74: fract_move(-1, 0, 0); replot=TRUE; break; // j
				case VK_UP:
				case 75: fract_move( 1, 0, 0); replot=TRUE; break; // k
				case 90: fract_move( 0, 0,-1); replot=TRUE; break; // z
				case 88: fract_move( 0, 0, 1); replot=TRUE; break; // x
				case 83: fract_style_toggle(); replot=TRUE; break; // s
				case VK_SPACE:
				case 65: fract_pilot_toggle(); replot=TRUE; break; // a
				case 68: // d
					sprintf(coordinates,"x (%f to %f); y (%f to %f)\ncenter: (%f,%f); width: %f",fract_bounds_get().xmin,fract_bounds_get().xmax,fract_bounds_get().ymin,fract_bounds_get().ymax,(fract_bounds_get().xmax+fract_bounds_get().xmin)/2,(fract_bounds_get().ymax+fract_bounds_get().ymin)/2,fract_bounds_get().xmax-fract_bounds_get().ymax);
					KillTimer(hWnd,1);
					MessageBox(hWnd,coordinates,"Coordinates",MB_OK);
					SetTimer(hWnd,1,120,NULL);
					Sleep(500);
				break;
				case 27: fract_reset(); replot=TRUE; break;
				default: break;
			}
		break;
		
		case WM_CREATE: // we're a new window, do init stuff
			fract_init();
			fract_plot();
			hdc = GetDC(hWnd);
			hbit= CreateCompatibleBitmap(hdc,DEFAULT_FRACT_WIDTH,DEFAULT_FRACT_HEIGHT);
			memdc = CreateCompatibleDC(hdc); // for the bitmap
			SelectObject(memdc,hbit);
			ready=TRUE;
			replot=TRUE;
			SetTimer(hWnd,1,120,NULL); // update baby!
		break;
		
		case WM_PAINT:
			hdc = BeginPaint(hWnd, &pstruct ); /* prepare window for painting*/
			SetMapMode(hdc,MM_ANISOTROPIC);
			SetWindowExtEx(hdc,DEFAULT_FRACT_WIDTH,DEFAULT_FRACT_HEIGHT,NULL);
			SetViewportExtEx(hdc,MYWIDTH,MYHEIGHT,NULL);
			BitBlt(hdc,0,0,DEFAULT_FRACT_WIDTH,DEFAULT_FRACT_HEIGHT,memdc,0,0,SRCCOPY);
			EndPaint(hWnd, &pstruct );	/* stop painting */
			UpdateWindow(hWnd);
		break;
		
		case WM_DESTROY:
			quit(hWnd);
		break;
	
		default:
		return( DefWindowProc( hWnd, messg, wParam, lParam ) );

	}
	if (replot) {
		width=(fract_size_get()).width;
		height=(fract_size_get()).height;
		fractal=fract_get();
		fract_plot();
		for (y=0,i=0;y<DEFAULT_FRACT_HEIGHT;y++) 
			for (x=0;x<DEFAULT_FRACT_WIDTH;x++,i+=3)
				SetPixelV(memdc,x,y,RGB(fractal[i],fractal[i+1],fractal[i+2]));
		InvalidateRect(hWnd,NULL,0); //right way to do it, ish... 0==don't erase
	}

	return( 0L );
}